import React from 'react'

export default function TorrentList(props){
  return (
    <div>
      <AddTorrent func={props.addTorrent} />
      <List torrents={props.torrentStore.torrents} />
    </div>
  );
}

class AddTorrent extends React.Component {
  constructor(props) {
    super(props);
    this.state = { url: '' };
    this.handleSubmit = this.handleSubmit.bind(this);
    this.handleChange = this.handleChange.bind(this);
  }
  handleChange(event) {
    this.setState({ url: event.target.value });
  }
  handleSubmit() {
    if (this.state.url === "") {
      return;
    }
    this.setState({ url: '' });
    this.props.func(this.state.url);
  }
  render() {
    return (
      <div className="row">
        <div className="col-xs-12 col-md-12">
          <form className="input-group" onSubmit={this.handleSubmit}>
            <input
              className="form-control"
              placeholder="Add torrent URL"
              onChange={this.handleChange}
              value={this.state.url}
            />
            <span className="input-group-btn">
              <button
                className="btn btn-primary"
                type="button"
                onClick={this.handleSubmit}
              >
                Add
              </button>
            </span>
          </form>
        </div>
      </div>
    );
  }
}

function List(props){
  if (props.torrents.length === 0) {
    return (
      <div className="row">
        <div className="col-xs-12 col-md-12">
          <h3>Torrents</h3>
          <div className="panel panel-default">
            <div className="panel-heading">No torrents</div>
          </div>
        </div>
      </div>
    );
  }
  return (
      <div className="row">
        <div className="col-xs-12 col-md-12">
          <h3>Torrents</h3>
          {props.torrents.map(function(el, index) {
            return (
              <Torrent key={index} data={el} />
            );
          })}
        </div>
      </div>
  );
}


function Torrent(props){
  const done = props.data.is_finished;
  var progressStyle = 'progress-bar progress-bar-warning';
  if (done) {
    progressStyle = 'progress-bar progress-bar-success';
  }
  var percentDone = props.data.percent_done;
  const started = (percentDone !== 0);
  if (started) {
    percentDone = Number(percentDone).toFixed(1) + '%';
  }

  var downloadedSize = prettySize(props.data.downloaded_size);
  var totalSize = prettySize(props.data.total_size);
  var downloadRate = prettySize(props.data.download_rate) + "/s";
  return (
    <div className="panel panel-default">
      <div className="panel-heading">{props.data.name}</div>
      <div className="panel-body">
        {started &&
            <div className="progress progress-striped">
              <div
                className={progressStyle}
                style={{width: percentDone}}>
              </div>
            </div>
        }
        {!started &&
            <p>Download not yet started</p>
        }
        {started &&
        <div>
          <p>{downloadedSize} / {totalSize} - {percentDone} - {downloadRate}</p>
        </div>
        }
      </div>
    </div>
  );
}

function prettySize(fileSizeInBytes) {
    var i = -1;
    var byteUnits = [' kB', ' MB', ' GB', ' TB', 'PB', 'EB', 'ZB', 'YB'];
    do {
        fileSizeInBytes = fileSizeInBytes / 1024;
        i++;
    } while (fileSizeInBytes > 1024);

    return Math.max(fileSizeInBytes, 0.1).toFixed(1) + byteUnits[i];
};
